 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: GlobalRequestFilterBridge
  * Author:   Louis
  * Date:     2021/3/30 22:09
  */
 package com.fr.plugin.j7814.request;

 import com.fanruan.api.decision.login.LoginKit;
 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.NetworkKit;
 import com.fanruan.api.util.StringKit;
 import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
 import com.fr.decision.webservice.utils.DecisionServiceConstants;
 import com.fr.decision.webservice.v10.login.LoginService;
 import com.fr.plugin.j7814.config.SsoConfig;
 import com.idsmanager.dingdang.jwt.DingdangUserRetriever;

 import javax.servlet.FilterChain;
 import javax.servlet.FilterConfig;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;

 /**
  * <Function Description><br>
  * <GlobalRequestFilterBridge>
  *
  * @author Louis
  * @since 1.0.0
  */
 public class GlobalRequestFilterBridge extends AbstractGlobalRequestFilterProvider {
     public static final String REMOTE_DESIGN = "/remote/design";
     public static final String RESOURCES_PATH = "/resources";
     public static final String VIEW_FORM = "/view/form";
     public static final String VIEW_REPORT = "/view/report";
     public static final String FILE_PATH = "/file";
     public static final String SYSTEM_INFO = "/system/info";
     public static final String MATERIALS_MIN_JS_MAP = "/materials.min.js.map";
     public static final String LOGIN_PATH = "/login";
     public static final String LOGIN_OTHER = "/login/";
     public static final String LOGOUT_PATH = "/logout";
     public static final String USER_LANGUAGE = "/v10/user/language";

     public static final String ID_TOKEN = "id_token";

     private SsoConfig config;

     /**
      * 过滤器名称
      *
      * @return
      */
     @Override
     public String filterName() {
         return "J7814Filter";
     }

     /**
      * 过滤规则
      *
      * @return
      */
     @Override
     public String[] urlPatterns() {
         return new String[]{"/decision/*"};
     }

     /**
      * 过滤器初始化
      *
      * @param filterConfig
      */
     @Override
     public void init(FilterConfig filterConfig) {
         this.config = SsoConfig.getInstance();
         super.init(filterConfig);
     }

     /**
      * 过滤器处理
      *
      * @param request
      * @param response
      * @param filterChain
      */
     @Override
     public void doFilter(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) {
         try {
             if (operation(request, response)) {
                 filterChain.doFilter(request, response);
             }
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }
     }

     /**
      * 用户验证登陆操作
      *
      * @param req
      * @param res
      * @throws Exception
      */
     private boolean operation(HttpServletRequest req, HttpServletResponse res) throws Exception {
         String pathInfo = (req.getPathInfo() != null) ? req.getPathInfo() : StringKit.EMPTY;
         LogKit.error("sso-GlobalRequestFilterBridge-operation-pathInfo:{}", pathInfo);
         if (pathInfo.startsWith(REMOTE_DESIGN) || pathInfo.startsWith(LOGIN_OTHER)
                 || StringKit.equals(LOGIN_PATH, pathInfo)
                 || pathInfo.startsWith(RESOURCES_PATH) || pathInfo.startsWith(LOGOUT_PATH)
                 || pathInfo.startsWith(SYSTEM_INFO) || pathInfo.startsWith(MATERIALS_MIN_JS_MAP)
                 || pathInfo.startsWith(USER_LANGUAGE) || pathInfo.startsWith(FILE_PATH)) {
             return true;
         }
//         if (pathInfo.startsWith(VIEW_REPORT) || pathInfo.startsWith(VIEW_FORM)) {
//             return true;
//         }
         // 已登录
         if (LoginService.getInstance().isLogged(req)) {
             return true;
         }
         String token = NetworkKit.getHTTPRequestParameter(req, ID_TOKEN);
         LogKit.error("sso-GlobalRequestFilterBridge-operation-token:{}", token);
         if (StringKit.isEmpty(token)) {
             return true;
         }
         String username = getUsername(token);
         LogKit.error("sso-GlobalRequestFilterBridge-operation-username:{}", username);
         if (StringKit.isEmpty(username)) {
             return true;
         }
         String tokenFR = LoginKit.login(req, res, username);
         req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, tokenFR);
         return true;
     }

     /**
      * 通过凭证获得username
      *
      * @param token
      * @return
      */
     private String getUsername(String token) throws Exception {
         DingdangUserRetriever retriever = new DingdangUserRetriever(token, this.config.getPublicKey());
         DingdangUserRetriever.User user = retriever.retrieve();
         if (user == null) {
             return StringKit.EMPTY;
         }
         return user.getUsername();
     }
 }